@@ -1,5 +1,5 @@
-/* $Id: compat.h,v 1.26 2004/06/20 21:53:57 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: compat.h,v 1.28 2005/09/10 16:48:01 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __compat_h_included
@@ -31,6 +31,16 @@
 
 /* substitute some features that might be missing on some platforms */
 
+#ifndef SHUT_RD
+#  define SHUT_RD 0
+#endif
+#ifndef SHUT_WR
+#  define SHUT_WR 1
+#endif
+#ifndef SHUT_RDWR
+#  define SHUT_RDWR 2
+#endif
+
 #ifndef MIN
 #  define MIN(x,y) ((x)<=(y)?(x):(y))
 #endif
@@ -71,12 +81,43 @@
 #endif
 
 
-#if HAVE_FORMAT_Z
-#  define F_Zd "%Zd"
-#  define F_Zu "%Zu"
-#else
+/* all unsigned */
+#if !defined(HAVE_BASIC_SIZE_T) || !HAVE_BASIC_SIZE_T
+#  undef HAVE_BASIC_SIZE_T
+#  define HAVE_BASIC_SIZE_T 5
+#endif
+#if HAVE_BASIC_SIZE_T==2
+#  define SIZET_MAX USHRT_MAX
+#  define SSIZET_MIN SHRT_MIN
+#  define SSIZET_MAX SHRT_MAX
+#  define F_Zd "%hd"
+#  define F_Zu "%hu"
+#elif HAVE_BASIC_SIZE_T==4
+#  define SIZET_MAX UINT_MAX
+#  define SSIZET_MIN INT_MIN
+#  define SSIZET_MAX INT_MAX
 #  define F_Zd "%d"
 #  define F_Zu "%u"
+#elif HAVE_BASIC_SIZE_T==6
+#  define SIZET_MAX ULONG_MAX
+#  define SSIZET_MIN LONG_MIN
+#  define SSIZET_MAX LONG_MAX
+#  define F_Zd "%ld"
+#  define F_Zu "%lu"
+#elif HAVE_BASIC_SIZE_T==8
+#  define SIZET_MAX ULLONG_MAX
+#  define SSIZET_MIN LLONG_MIN
+#  define SSIZET_MAX LLONG_MAX
+#  define F_Zd "%Ld"
+#  define F_Zu "%Lu"
+#else
+#  error "HAVE_BASIC_SIZE_T is out of range:" HAVE_BASIC_SIZE_T
+#endif
+#if HAVE_FORMAT_Z
+#  undef F_Zd
+#  undef F_Zu
+#  define F_Zd "%Zd"
+#  define F_Zu "%Zu"
 #endif
 
 
