@@ -1,5 +1,5 @@
-/* $Id: filan_main.c,v 1.14 2004/12/04 21:40:02 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: filan_main.c,v 1.16 2005/03/18 20:26:26 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 const char copyright[] = "filan by Gerhard Rieger - see http://www.dest-unreach.org/socat/";
@@ -108,15 +108,11 @@ int main(int argc, const char *argv[]) {
          break;
       case '\0': break;
       default:
-	 {
-	    char l[2];
-	    l[0] = '0'+E_FATAL; l[1] = '\0';
-	    diag_set('e', l);
-	    Error1("unknown option %s", arg1[0]);
+	 diag_set_int('e', E_FATAL);
+	 Error1("unknown option %s", arg1[0]);
 #if WITH_HELP
-	    filan_usage(stderr);
+	 filan_usage(stderr);
 #endif
-	 }
 	 exit(1);
       }
 #if 0
