@@ -1,5 +1,5 @@
 #! /bin/bash
-# $Id: readline.sh,v 1.2 2004/06/06 17:33:22 gerhard Exp $
+# $Id: readline.sh,v 1.3 2004/08/25 15:52:59 gerhard Exp $
 # Copyright Gerhard Rieger 2003-2004
 # Published under the GNU General Public License V.2, see file COPYING
 
@@ -17,8 +17,8 @@ done
 
 PROGRAM="$@"
 if [ "$withhistfile" ]; then
-    HISTFILE="$HOME/.$2_history"
-    $HISTOPT=",history=$HISTFILE"
+    HISTFILE="$HOME/.$1_history"
+    HISTOPT=",history=$HISTFILE"
 else
     HISTOPT=
 fi
