@@ -1,4 +1,4 @@
-/* $Id: socat.c,v 1.92 2005/03/13 12:12:26 gerhard Exp $ */
+/* $Id: socat.c,v 1.94 2005/09/04 09:39:51 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -20,6 +20,7 @@
 #include "filan.h"
 #include "xio.h"
 #include "xioopts.h"
+#include "xiolockfile.h"
 
 
 /* command line options */
@@ -34,6 +35,7 @@ struct {
    char logopt;		/* y..syslog; s..stderr; f..file; m..mixed */
    bool lefttoright;	/* first addr ro, second addr wo */
    bool righttoleft;	/* first addr wo, second addr ro */
+   xiolock_t lock;	/* a lock file */
 } socat_opts = {
    8192,	/* bufsiz */
    false,	/* verbose */
@@ -44,7 +46,8 @@ struct {
    0,		/* strictopts */
    's',		/* logopt */
    false,	/* lefttoright */
-   false	/* righttoleft */
+   false,	/* righttoleft */
+   { NULL, 0 },	/* lock */
 };
 
 void socat_usage(FILE *fd);
@@ -58,6 +61,10 @@ static int socat_sigchild(struct single *file);
 void lftocrlf(char **in, ssize_t *len, size_t bufsiz);
 void crlftolf(char **in, ssize_t *len, size_t bufsiz);
 
+static int socat_lock(void);
+static void socat_unlock(void);
+static int socat_newchild(pid_t pid);
+
 static const char socatversion[] =
 #include "./VERSION"
       ;
@@ -69,6 +76,8 @@ const char copyright_openssl[] = "This product includes software developed by th
 const char copyright_ssleay[] = "This product includes software written by Tim Hudson (tjh@cryptsoft.com)";
 #endif
 
+bool havelock;
+
 
 int main(int argc, const char *argv[]) {
    const char **arg1, *a;
@@ -76,6 +85,7 @@ int main(int argc, const char *argv[]) {
    double rto;
    int i, argc0, result;
    struct utsname ubuf;
+   int lockrc;
 
    diag_set('p', strchr(argv[0], '/') ? strrchr(argv[0], '/')+1 : argv[0]);
    xiosetopt('p', "!!");
@@ -155,6 +165,33 @@ int main(int argc, const char *argv[]) {
       case 'u': socat_opts.lefttoright = true; break;
       case 'U': socat_opts.righttoleft = true; break;
       case 'g': xioopts_ignoregroups = true; break;
+      case 'L': if (socat_opts.lock.lockfile)
+	     Error("only one -L and -W option allowed");
+	 if (arg1[0][2]) {
+	    socat_opts.lock.lockfile = *arg1+2;
+	 } else {
+	    ++arg1, --argc;
+	    if ((socat_opts.lock.lockfile = *arg1) == NULL) {
+	       Error("option -L requires an argument; use option \"-h\" for help");
+	       socat_usage(stderr); Exit(1);
+	    }
+	 }
+	 break;
+      case 'W': if (socat_opts.lock.lockfile)
+	    Error("only one -L and -W option allowed");
+	 if (arg1[0][2]) {
+	    socat_opts.lock.lockfile = *arg1+2;
+	 } else {
+	    ++arg1, --argc;
+	    if ((socat_opts.lock.lockfile = *arg1) == NULL) {
+	       Error("option -W requires an argument; use option \"-h\" for help");
+	       socat_usage(stderr); Exit(1);
+	    }
+	 }
+	 socat_opts.lock.waitlock = true;
+	 socat_opts.lock.intervall.tv_sec  = 1;
+	 socat_opts.lock.intervall.tv_nsec = 0;
+	 break;
       case '\0':
       case ',':
       case ':': break;	/* this "-" is a variation of STDIO */
@@ -211,6 +248,19 @@ int main(int argc, const char *argv[]) {
    Signal(SIGFPE, socat_signal);
    Signal(SIGSEGV, socat_signal);
    Signal(SIGTERM, socat_signal);
+
+   /* set xio hooks */
+   xiohook_newchild = &socat_newchild;
+
+   if (lockrc = socat_lock()) {
+      /* =0: goon; >0: locked; <0: error, printed in sub */
+      if (lockrc > 0)
+	 Error1("could not obtain lock \"%s\"", socat_opts.lock.lockfile);
+      Exit(1);
+   }
+
+   Atexit(socat_unlock);
+
    result = socat(arg1[0], arg1[1]);
    Notice1("exiting with status %d", result);
    Exit(result);
@@ -247,6 +297,8 @@ void socat_usage(FILE *fd) {
    fputs("      -u     unidirectional mode (left to right)\n", fd);
    fputs("      -U     unidirectional mode (right to left)\n", fd);
    fputs("      -g     do not check option groups\n", fd);
+   fputs("      -L <lockfile>  try to obtain lock, or fail\n", fd);
+   fputs("      -W <lockfile>  try to obtain lock, or wait\n", fd);
 }
 
 
@@ -545,11 +597,11 @@ int childleftdata(xiofile_t *xfd) {
 
       if (retval < 0) {
 #if HAVE_FDS_BITS
-	 Error5("select(%d, &%lu, &%lu, &%lu, {0}): %s",
+	 Error5("select(%d, &0x%lx, &0x%lx, &0x%lx, {0}): %s",
 		FOPEN_MAX, in.fds_bits[0], out.fds_bits[0],
 		expt.fds_bits[0], strerror(errno));
 #else
-	 Error5("select(%d, &%lu, &%lu, &%lu, {0}): %s",
+	 Error5("select(%d, &0x%lx, &0x%lx, &0x%lx, {0}): %s",
 		FOPEN_MAX, in.__fds_bits[0], out.__fds_bits[0],
 		expt.__fds_bits[0], strerror(errno));
 #endif
@@ -574,7 +626,8 @@ bool maywr1;		/* sock1 can be written to, according to select() */
 bool maywr2;		/* sock2 can be written to, according to select() */
 
 /* here we come when the sockets are opened (in the meaning of C language),
-   and their options are set/applied */
+   and their options are set/applied
+   returns -1 on error or 0 on success */
 int _socat(void) {
    fd_set in, out, expt;
    struct timeval timeout, *to;
@@ -696,12 +749,12 @@ int _socat(void) {
 
       if (retval < 0) {
 #if HAVE_FDS_BITS
-	    Error7("select(%d, &%lu, &%lu, &%lu, %s%lu): %s",
+	    Error7("select(%d, &0x%lx, &0x%lx, &0x%lx, %s%lu): %s",
 		   FOPEN_MAX, in.fds_bits[0], out.fds_bits[0],
 		   expt.fds_bits[0], to?"&":"NULL/", to?to->tv_sec:0,
 		   strerror(errno));
 #else
-	    Error7("select(%d, &%lu, &%lu, &%lu, %s%lu): %s",
+	    Error7("select(%d, &0x%lx, &0x%lx, &0x%lx, %s%lu): %s",
 		   FOPEN_MAX, in.__fds_bits[0], out.__fds_bits[0],
 		   expt.__fds_bits[0], to?"&":"NULL/", to?to->tv_sec:0,
 		   strerror(errno));
@@ -739,11 +792,16 @@ int _socat(void) {
 	     < 0) {
 	    if (errno != EAGAIN) {
 	       closing = MAX(closing, 1);
+	       Notice("socket 1 to socket 2 is in error");
+	       if (socat_opts.lefttoright) {
+		  break;
+	       }
 	    }
 	 } else if (bytes1 > 0) {
 	    wasaction = 1;
 	 }
-      } else {	/* (bytes == 0)  handled later */
+	 /* (bytes1 == 0)  handled later */
+      } else {
 	 bytes1 = -1;
       }
 
@@ -753,10 +811,15 @@ int _socat(void) {
 	     < 0) {
 	    if (errno != EAGAIN) {
 	       closing = MAX(closing, 1);
+	       Notice("socket 2 to socket 1 is in error");
+	       if (socat_opts.righttoleft) {
+		  break;
+	       }
 	    }
 	 } else if (bytes2 > 0) {
 	    wasaction = 1;
 	 }
+	 /* (bytes2 == 0)  handled later */
       } else {
 	 bytes2 = -1;
       }
@@ -790,7 +853,6 @@ int _socat(void) {
 	    }
 	 }
       }
-
    }
 
    /* close everything that's still open */
@@ -923,6 +985,9 @@ int xiotransfer(xiofile_t *inpipe, xiofile_t *outpipe,
 
 	    writt = xiowrite(outpipe, *buff, bytes);
 	    if (writt < 0) {
+	       if (errno == EPIPE) {
+		  return 0;	/* can no longer write; handle like EOF */
+	       }
 	       return -1;
 	    } else {
 	       Info3("transferred "F_Zu" bytes from %d to %d",
@@ -1032,7 +1097,63 @@ static int socat_sigchild(struct single *file) {
    if (file->ignoreeof && !closing) {
       sleepon = 1;
    } else {
-      file->eof = 1; closing = 1;
+      file->eof = MAX(file->eof, 1);
+      closing = 1;
    }
    return 0;
 }
+
+static int socat_lock(void) {
+   int lockrc;
+
+#if 1
+   if ((lockrc = xiolock(&socat_opts.lock)) < 0) {
+      return -1;
+   }
+   if (lockrc == 0) {
+      havelock = true;
+   }
+   return lockrc;
+#else
+   if (socat_opts.lock.lockfile) {
+      if ((lockrc = xiolock(socat_opts.lock.lockfile)) < 0) {
+	 /*Error1("error with lockfile \"%s\"", socat_opts.lock.lockfile);*/
+	 return -1;
+      }
+      if (lockrc) {
+	 return 1;
+      }
+      havelock = true;
+      /*0 Info1("obtained lock \"%s\"", socat_opts.lock.lockfile);*/
+   }
+
+   if (socat_opts.lock.waitlock) {
+      if (xiowaitlock(socat_opts.lock.waitlock, socat_opts.lock.intervall)) {
+	 /*Error1("error with lockfile \"%s\"", socat_opts.lock.lockfile);*/
+	 return -1;
+      } else {
+	 havelock = true;
+	 /*0 Info1("obtained lock \"%s\"", socat_opts.lock.waitlock);*/
+      }
+   }
+   return 0;
+#endif
+}
+
+static void socat_unlock(void) {
+   if (!havelock)  return;
+   if (socat_opts.lock.lockfile) {
+      if (Unlink(socat_opts.lock.lockfile) < 0) {
+	 Warn1("unlink(\"%s\"): %s", socat_opts.lock.lockfile);
+      } else {
+	 Info1("released lock \"%s\"", socat_opts.lock.lockfile);
+      }
+   }
+}
+
+/* this is a callback function that may be called by the newchild hook of xio
+ */
+static int socat_newchild(pid_t pid) {
+   havelock = false;
+   return 0;
+}
