@@ -1,5 +1,5 @@
-/* $Id: sslcls.h,v 1.6 2004/10/24 13:28:55 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: sslcls.h,v 1.7 2005/03/12 11:09:37 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __sslcls_h_included
@@ -27,6 +27,7 @@ int sycSSL_CTX_use_certificate_chain_file(SSL_CTX *ctx, const char *file);
 int sycSSL_CTX_use_PrivateKey_file(SSL_CTX *ctx, const char *file, int type);
 void sycSSL_CTX_set_verify(SSL_CTX *ctx, int mode,
 			   int (*verify_callback)(int, X509_STORE_CTX *));
+int sycSSL_CTX_set_tmp_dh(SSL_CTX *ctx, DH *dh);
 int sycSSL_CTX_set_cipher_list(SSL_CTX *ctx, const char *str);
 int sycSSL_set_cipher_list(SSL *ssl, const char *str);
 long sycSSL_get_verify_result(SSL *ssl);
@@ -41,6 +42,10 @@ void sycSSL_CTX_free(SSL_CTX *ctx);
 void sycSSL_free(SSL *ssl);
 int sycRAND_egd(const char *path);
 
+DH *sycPEM_read_bio_DHparams(BIO *bp, DH **x, pem_password_cb *cb, void *u);
+
+BIO *sycBIO_new_file(const char *filename, const char *mode);
+
 #endif /* WITH_OPENSSL */
 
 #else /* !WITH_SYCLS */
@@ -64,6 +69,7 @@ int sycRAND_egd(const char *path);
 #define sycSSL_CTX_use_certificate_chain_file(c,f) SSL_CTX_use_certificate_chain_file(c,f)
 #define sycSSL_CTX_use_PrivateKey_file(c,f,t) SSL_CTX_use_PrivateKey_file(c,f,t)
 #define sycSSL_CTX_set_verify(c,m,v) SSL_CTX_set_verify(c,m,v)
+#define sycSSL_CTX_set_tmp_dh(c,d) SSL_CTX_set_tmp_dh(c,d)
 #define sycSSL_CTX_set_cipher_list(c,s) SSL_CTX_set_cipher_list(c,s)
 #define sycSSL_set_cipher_list(s,t) SSL_set_cipher_list(s,t)
 #define sycSSL_get_verify_result(s) SSL_get_verify_result(s)
@@ -78,6 +84,10 @@ int sycRAND_egd(const char *path);
 #define sycSSL_free(s) SSL_free(s)
 #define sycRAND_egd(p) RAND_egd(p)
 
+#define sycPEM_read_bio_DHparams(b,x,p,u) PEM_read_bio_DHparams(b,x,p,u)
+
+#define sycBIO_new_file(f,m) BIO_new_file(f,m)
+
 #endif /* WITH_OPENSSL */
 
 #endif /* !WITH_SYCLS */
