@@ -1,4 +1,4 @@
-/* $Id: sycls.c,v 1.65 2005/03/12 10:55:38 gerhard Exp $ */
+/* $Id: sycls.c,v 1.66 2005/08/18 19:44:49 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -780,6 +780,16 @@ int Kill(pid_t pid, int sig) {
    return retval;
 }
 
+int Link(const char *oldpath, const char *newpath) {
+   int retval, _errno;
+   Debug2("link(\"%s\", \"%s\")", oldpath, newpath);
+   retval = link(oldpath, newpath);
+   _errno = errno;
+   Debug1("link() -> %d", retval);
+   errno = _errno;
+   return retval;
+}
+
 int Execvp(const char *file, char *const argv[]) {
    int result, _errno;
    Debug2("execvp(%s, %s, ...)", file, argv[0]);
@@ -1319,6 +1329,15 @@ void Exit(int status) {
    exit(status);
 }
 
+int Mkstemp(char *template) {
+   int result, _errno;
+   Debug1("mkstemp(\"%s\")", template);
+   result = mkstemp(template);
+   _errno = errno;
+   Info2("mkstemp({%s}) -> %d", template, result);
+   errno = _errno;
+   return result;
+}
 
 #if WITH_READLINE
 
