@@ -1,6 +1,6 @@
 #! /bin/bash
-# $Id: test.sh,v 1.95 2005/03/18 21:22:25 gerhard Exp $
-# Copyright Gerhard Rieger 2001-2005
+# $Id: test.sh,v 1.101 2006/01/29 16:36:02 gerhard Exp $
+# Copyright Gerhard Rieger 2001-2006
 # Published under the GNU General Public License V.2, see file COPYING
 
 # perform lots of tests on socat
@@ -396,7 +396,7 @@ for addr in proxy; do
     done
     echo $OPTS
     # prepare dummy server
-    $SOCAT tcp-l:$PORT,reuseaddr,crlf exec:"bash proxyecho.sh" || echo "cannot start proxyecho.sh" >&2 &
+    $SOCAT tcp-l:$PORT,reuseaddr,crlf exec:"/bin/bash proxyecho.sh" || echo "cannot start proxyecho.sh" >&2 &
 	pid=$!
 	sleep 1
     for o in $(filloptionvalues $OPTS|tr ',' ' '); do
@@ -1165,7 +1165,7 @@ runsip6 () {
     Linux) l=$(/sbin/ifconfig |grep 'inet6 addr: ::1/') ;;
     NetBSD)l=$(/sbin/ifconfig -a |grep ' inet6 ::1 ');;
     OSF1)  l=$(/sbin/ifconfig -a |grep ' inet6 ') ;;
-    SunOS) l=$(ifconfig -a |grep ' inet6 ') ;;
+    SunOS) l=$(/sbin/ifconfig -a |grep 'inet6 ') ;;
     *)     l=$(/sbin/ifconfig -a |grep ' ::1[^:0-9A-Fa-f]') ;;
     esac
     [ -z "$l" ] && return 1    
@@ -1189,13 +1189,17 @@ waittcp4port () {
     [ "$timeout" ] || timeout=5
     while [ $timeout -gt 0 ]; do
 	case "$UNAME" in
-	Linux)   l=$(netstat -an |grep '^tcp .* .*[0-9*]:'$port' .* LISTEN') ;;
+	Linux)   l=$(netstat -n -t -l |grep '^tcp .* .*[0-9*]:'$port' .* LISTEN') ;;
 	FreeBSD) l=$(netstat -an |grep '^tcp4.* .*[0-9*]\.'$port' .* \*\.\* .* LISTEN') ;;
-	Darwin)  l=$(netstat -an |grep '^tcp4.* .*[0-9*]\.'$port' .* \*\.\* .* LISTEN') ;;
+	Darwin) case "$(uname -r)" in
+		[1-5]*) l=$(netstat -an |grep '^tcp.* .*[0-9*]\.'$port' .* \*\.\* .* LISTEN') ;;
+		*) l=$(netstat -an |grep '^tcp4.* .*[0-9*]\.'$port' .* \*\.\* .* LISTEN') ;;
+		esac ;;
 	AIX)	 l=$(netstat -an |grep '^tcp[^6]       0      0 .*[*0-9]\.'$port' .* LISTEN$') ;;
-	SunOS)   l=$(netstat -an -f inet |grep '.*[1-9*]\.'$port' .*\*                0 .* LISTEN') ;;
+	SunOS)   l=$(netstat -an -f inet -P tcp |grep '.*[1-9*]\.'$port' .*\*                0 .* LISTEN') ;;
 	HP-UX)   l=$(netstat -an |grep '^tcp        0      0  .*[0-9*]\.'$port' .* LISTEN$') ;;
 	OSF1)    l=$(/usr/sbin/netstat -an |grep '^tcp        0      0  .*[0-9*]\.'$port' [ ]*\*\.\* [ ]*LISTEN') ;;
+	CYGWIN*) l=$(netstat -an -p TCP |grep '^  TCP    [0-9.]*:'$port' .* LISTENING') ;;
  	*)       l=$(netstat -an |grep -i 'tcp .*[0-9*][:.]'$port' .* listen') ;;
 	esac
 	[ \( \( $logic -ne 0 \) -a -n "$l" \) -o \
@@ -1218,13 +1222,16 @@ waitudp4port () {
     [ "$timeout" ] || timeout=5
     while [ $timeout -gt 0 ]; do
 	case "$UNAME" in
-	Linux)   l=$(netstat -an |grep '^udp .* .*[0-9*]:'$port' [ ]*0\.0\.0\.0:\*') ;;
+	Linux)   l=$(netstat -n -u -l |grep '^udp .* .*[0-9*]:'$port' [ ]*0\.0\.0\.0:\*') ;;
 	FreeBSD) l=$(netstat -an |grep '^udp4[6 ] .*[0-9*]\.'$port' .* \*\.\*') ;;
 	NetBSD)  l=$(netstat -an |grep '^udp .*[0-9*]\.'$port' [ ]* \*\.\*') ;;
 	OpenBSD) l=$(netstat -an |grep '^udp .*[0-9*]\.'$port' [ ]* \*\.\*') ;;
-	Darwin)  l=$(netstat -an |grep '^udp4.* .*[0-9*]\.'$port' .* \*\.\* .*') ;;
+	Darwin) case "$(uname -r)" in
+		[1-5]*) l=$(netstat -an |grep '^udp.* .*[0-9*]\.'$port' .* \*\.\*') ;;
+		*) l=$(netstat -an |grep '^udp4.* .*[0-9*]\.'$port' .* \*\.\* .*') ;;
+		esac ;;
 	AIX)	 l=$(netstat -an |grep '^udp[4 ]       0      0 .*[*0-9]\.'$port' .* \*\.\*[ ]*$') ;;
-	SunOS)   l=$(netstat -an -f inet |grep '.*[1-9*]\.'$port' [ ]*Idle') ;;
+	SunOS)   l=$(netstat -an -f inet -P udp |grep '.*[1-9*]\.'$port' [ ]*Idle') ;;
 	HP-UX)   l=$(netstat -an |grep '^udp        0      0  .*[0-9*]\.'$port' .* \*\.\* ') ;;
 	OSF1)    l=$(/usr/sbin/netstat -an |grep '^udp        0      0  .*[0-9*]\.'$port' [ ]*\*\.\*') ;;
  	*)       l=$(netstat -an |grep -i 'udp .*[0-9*][:.]'$port' ') ;;
@@ -1253,7 +1260,7 @@ waittcp6port () {
 	FreeBSD) l=$(netstat -an |grep -i 'tcp[46][6 ] .*[0-9*][:.]'$port' .* listen') ;;
 	OpenBSD) l=$(netstat -an |grep -i 'tcp6 .*[0-9*][:.]'$port' .* listen') ;;
 	AIX)	 l=$(netstat -an |grep '^tcp[6 ]       0      0 .*[*0-9]\.'$port' .* LISTEN$') ;;
-	#SunOS)   l=$(netstat -an -f inet6 |grep '.*[1-9*]\.'$port' .*\*                0 .* LISTEN') ;;
+	SunOS)   l=$(netstat -an -f inet6 -P tcp |grep '.*[1-9*]\.'$port' .*\* [ ]* 0 .* LISTEN') ;;
 	#OSF1)    l=$(/usr/sbin/netstat -an |grep '^tcp6       0      0  .*[0-9*]\.'$port' [ ]*\*\.\* [ ]*LISTEN') /*?*/;;
  	*)       l=$(netstat -an |grep -i 'tcp6 .*:'$port' .* listen') ;;
 	esac
@@ -1282,7 +1289,7 @@ waitudp6port () {
     	OpenBSD) l=$(netstat -an |grep '^udp6 .*[0-9*]\.'$port' [ ]* \*\.\*') ;;
 	#Darwin)  l=$(netstat -an |grep '^udp6.* .*[0-9*]\.'$port' .* \*\.\* ') ;;
 	AIX)	 l=$(netstat -an |grep '^udp[6 ]       0      0 .*[*0-9]\.'$port' .* \*\.\*[ ]*$') ;;
-	#SunOS)   l=$(netstat -an -f inet6 |grep '.*[1-9*]\.'$port' [ ]*Idle') ;;
+	SunOS)   l=$(netstat -an -f inet6 -P udp |grep '.*[1-9*]\.'$port' [ ]*Idle') ;;
 	#HP-UX)   l=$(netstat -an |grep '^udp        0      0  .*[0-9*]\.'$port' ') ;;
 	#OSF1)    l=$(/usr/sbin/netstat -an |grep '^udp6       0      0  .*[0-9*]\.'$port' [ ]*\*\.\*') ;;
  	*)       l=$(netstat -an |grep -i 'udp .*[0-9*][:.]'$port' ') ;;
@@ -1677,7 +1684,7 @@ case "$TESTS" in
 TEST="$NAME: echo via connection to UNIX domain socket"
 tf="$td/test$N.stdout"
 te="$td/test$N.stderr"
-ts="$td/socket$N"
+ts="$td/test$N.socket"
 tdiff="$td/test$N.diff"
 da=$(date)
 CMD1="$SOCAT $opts UNIX-LISTEN:$ts PIPE"
@@ -1691,10 +1698,10 @@ if [ $? -ne 0 ]; then
    $PRINTF "$FAILED: $SOCAT:\n"
    echo "$CMD1 &"
    echo "$CMD2"
-   cat $te
-elif ! echo "$da" |diff - "$tf" >$tdiff; then
+   cat "$te"
+elif ! echo "$da" |diff - "$tf" >"$tdiff"; then
    $PRINTF "$FAILED: diff:\n"
-   cat $tdiff
+   cat "$tdiff"
 else
    $PRINTF "$OK\n"
    if [ -n "$debug" ]; then cat $te; fi
@@ -1817,13 +1824,19 @@ CMD="$SOCAT $opts $tp!!/dev/null /dev/null,ignoreeof!!$tf"
 printf "test %2d $TEST... " $N
 #mknod $tp p	# no mknod p on FreeBSD
 mkfifo $tp
-#$CMD >$tf 2>"$te" &
-($CMD >$tf 2>"$te" || rm -f "$tp") &
+$CMD >$tf 2>"$te" &
+#($CMD >$tf 2>"$te" || rm -f "$tp") 2>/dev/null &
 bg=$!	# background process id
 usleep $MICROS
+if [ ! -p "$tp" ]; then
+    $PRINTF "$FAILED: $SOCAT:\n"
+    echo "$CMD"
+    cat "$te"
+else
 echo "$da" >"$tp"
 # Solaris needs more time:
 sleep 1
+kill "$bg" 2>/dev/null
 if ! echo "$da" |diff - "$tf" >"$tdiff"; then
     if [ -s "$te" ]; then
 	$PRINTF "$FAILED: $SOCAT:\n"
@@ -1837,6 +1850,8 @@ else
    $PRINTF "$OK\n"
    if [ -n "$debug" ]; then cat $te; fi
 fi
+fi
+wait
 esac
 N=$((N+1))
 
@@ -2204,7 +2219,7 @@ te="$td/test$N.stderr"
 tdiff="$td/test$N.diff"
 da="$(date)" da="$da$($ECHO '\r')"
 #CMD2="$SOCAT tcp-l:$PORT,crlf system:\"read; read; $ECHO \\\"HTTP/1.0 200 OK\n\\\"; cat\""
-CMD2="$SOCAT tcp-l:$PORT,reuseaddr,crlf exec:\"bash proxyecho.sh\""
+CMD2="$SOCAT tcp-l:$PORT,reuseaddr,crlf exec:\"/bin/bash proxyecho.sh\""
 CMD="$SOCAT $opts - proxy:$LOCALHOST:127.0.0.1:1000,proxyport=$PORT"
 printf "test %2d $TEST... " $N
 eval "$CMD2 2>${te}1 &"
@@ -2333,7 +2348,7 @@ te="$td/test$N.stderr"
 tdiff="$td/test$N.diff"
 da="$(date)" da="$da$($ECHO '\r')"
 #CMD2="$SOCAT tcp-l:$PORT,crlf system:\"read; read; $ECHO \\\"HTTP/1.0 200 OK\n\\\"; cat\""
-CMD2="$SOCAT tcp-l:$PORT,reuseaddr,crlf exec:\"bash proxyecho.sh -w 2\""
+CMD2="$SOCAT tcp-l:$PORT,reuseaddr,crlf exec:\"/bin/bash proxyecho.sh -w 2\""
 CMD="$SOCAT $opts - proxy:$LOCALHOST:127.0.0.1:1000,proxyport=$PORT"
 printf "test %2d $TEST... " $N
 eval "$CMD2 2>${te}1 &"
@@ -2785,7 +2800,7 @@ testserversec () {
 	$PRINTF "$NO_RESULT (ph.1 server not working):\n"
 	echo "$SOCAT $opts \"$arg1,$secopt0\" echo &"
 	cat "${te}1"
-	return
+	wait; return
     fi
     # now use client
     (echo "$da"; sleep $T) |$SOCAT $opts - "$arg2" >"$tf" 2>"${te}2"
@@ -2798,7 +2813,7 @@ testserversec () {
 	cat "${te}1"
 	echo "$SOCAT $opts - \"$arg2\""
 	cat "${te}2"
-	return
+	wait; return
     elif echo "$da" |diff - "$tf" >"$tdiff1" 2>&1; then
 	:	# function without security is ok, go on
     else
@@ -2808,13 +2823,13 @@ testserversec () {
 	echo "$SOCAT $opts - $arg2"
 	cat "${te}2"
 	cat "$tdiff1"
-	return
+	wait; return
     fi
 
     # then: with security
     if [ "$port" ] && ! wait${proto}${ipvers}port $port 0; then
 	$PRINTF "$NO_RESULT (ph.1 port remains in use)\n"
-	return
+	wait; return
     fi
 
     # start server
@@ -2823,7 +2838,7 @@ testserversec () {
     if [ "$port" ] && ! wait${proto}${ipvers}port $port 1; then
 	kill $spid 2>/dev/null
 	$PRINTF "$NO_RESULT (ph.2 server not working)\n"
-	return
+	wait; return
     fi
     # now use client
     (echo "$da"; sleep $T) |$SOCAT $opts - "$arg2" >"$tf" 2>"${te}4"
@@ -2864,6 +2879,7 @@ testserversec () {
 	$PRINTF "$OK\n"
 	if [ -n "$debug" ]; then cat $te; fi
     fi
+    wait
 #set +vx
 }
 
@@ -3142,7 +3158,7 @@ waitfile "$tp"
 $SOCAT $opts -lpsocat2 file:/dev/null unix:"$ts" 2>"$te2"
 # if pty is blocking, first socat is still active and we get a connection now
 #((echo "$da"; sleep 2) |$SOCAT -lpsocat3 $opts - file:"$tp",$PTYOPTS2 >"$tf" 2>"$te3") &
-((waitfile "$ts"; echo "$da"; sleep 1) |$SOCAT -lpsocat3 $opts - file:"$tp",$PTYOPTS2 >"$tf" 2>"$te3") &
+( (waitfile "$ts"; echo "$da"; sleep 1) |$SOCAT -lpsocat3 $opts - file:"$tp",$PTYOPTS2 >"$tf" 2>"$te3") &
 waitfile "$ts"
 # but we need an echoer on the socket
 $SOCAT $opts -lpsocat4 unix:"$ts" echo 2>"$te4"
@@ -3272,12 +3288,21 @@ PORT=$((PORT+1))
 N=$((N+1))
 
 
-signum() {
+# derive signal number from signal name
+# kill -l should provide the info
+signum () {
+  if [ "$KSH_VERSION" ]; then
+    # we expect:
+    for i in $(kill -l); do echo $i; done |grep -n $1 |cut -d: -f1
+  else
+    # expect:
+    # " 1) SIGHUP       2) SIGINT       3) SIGQUIT      4) SIGILL"
     signam="$1"
     kill -l </dev/null |
     while read l; do printf "%s %s\n%s %s\n%s %s\n%s %s\n" $l; done |
     grep -e "SIG$signam\$" |
     cut -d ')' -f 1
+  fi
 }
 
 # problems with QUIT, INT (are blocked in system() )
@@ -3293,8 +3318,17 @@ tp="$td/test$N.pid"
 $PRINTF "test %2d $TEST... " $N
 (sleep 1; kill -$SIG $(cat "$tpp")) &
 # a simple "system:echo $PPID..." does not work on NetBSD, OpenBSD
-$SOCAT $opts echo system:'exec /bin/bash -c "echo \$PPID '">$tpp"'; echo \$$ '">$tp; read x\"",nofork 2>"$te"; stat=$?
-sleep 1; kill $(cat $tp)
+#$SOCAT $opts echo system:'exec /bin/bash -c "echo \$PPID '">$tpp"'; echo \$$ '">$tp; read x\"",nofork 2>"$te"; stat=$?
+tsh="$td/test$N.sh"
+cat <<EOF >"$tsh"
+#! /bin/bash
+echo \$PPID >"$tpp"
+echo \$\$ >"$tp"
+read x
+EOF
+chmod a+x "$tsh"
+$SOCAT $opts echo system:"exec \"$tsh\"",pty,setsid,nofork 2>"$te"; stat=$?
+sleep 1; kill -INT $(cat $tp)
 wait
 if [ "$stat" -eq $((128+$SIG)) ]; then
     $PRINTF "$OK\n"
@@ -3308,6 +3342,96 @@ N=$((N+1))
 done
 
 
+NAME=READBYTES
+#set -vx
+case "$TESTS" in
+*%$NAME%*|*%FUNCTIONS%*)
+TEST="$NAME: restrict reading from file with bytes option"
+if false; then
+    $PRINTF "test %2d $TEST... ${YELLOW}$(echo $feat| tr 'a-z' 'A-Z') not available${NORMAL}\n" $N
+else
+tr="$td/test$N.ref"
+ti="$td/test$N.in"
+to="$td/test$N.out"
+te="$td/test$N.err"
+tdiff="$td/test$N.diff"
+da="$(date)" da="$da$($ECHO '\r')"
+# the feature that we really want to test is in the readline.sh script:
+CMD="$SOCAT $opts -u open:$ti,readbytes=100 -"
+printf "test %2d $TEST... " $N
+rm -f "$tf" "$ti" "$to"
+#
+echo "AAAAAAAAAAAAAAAAAAAAAAAA
+AAAAAAAAAAAAAAAAAAAAAAAA
+AAAAAAAAAAAAAAAAAAAAAAAA
+AAAAAAAAAAAAAAAAAAAAAAAA" >"$tr"	# 100 bytes
+cat "$tr" "$tr" >"$ti"			# 200 bytes
+$CMD >"$to" 2>"$te"
+if ! diff "$tr" "$to" >"$tdiff" 2>&1; then
+    $PRINTF "$FAILED: $SOCAT:\n"
+    echo "$CMD"
+    cat "$te"
+    cat "$tdiff"
+else
+   $PRINTF "$OK\n"
+   if [ -n "$debug" ]; then cat $te; fi
+fi
+fi
+esac
+N=$((N+1))
+
+
+NAME=UNIXLISTENFORK
+case "$TESTS" in
+*%$NAME%|*%FUNCTIONS%*)
+TEST="$NAME: UNIX socket keeps listening after child died"
+if false; then
+    :
+else
+ts="$td/test$N.socket"
+tf="$td/test$N.stdout"
+te="$td/test$N.stderr"
+tdiff="$td/test$N.diff"
+da1="test$N $(date) $RANDOM"
+da2="test$N $(date) $RANDOM"
+#establish a listening and forking unix socket in background
+SRV="$SOCAT $opts -lpserver UNIX-LISTEN:\"$ts\",fork PIPE"
+#make a first and a second connection
+CLI="$SOCAT $opts -lpclient - UNIX-CONNECT:\"$ts\""
+$PRINTF "test %2d $TEST... " $N
+eval "$SRV 2>${te}s &"
+pids=$!
+waitfile "$ts"
+echo "$da1" |eval "$CLI" >"${tf}1" 2>"${te}1"
+if [ $? -ne 0 ]; then
+    kill "$pids"
+    $PRINTF "$NO_RESULT (first conn failed):\n"
+    echo "$SRV &"
+    echo "$CLI"
+    cat "${te}s" "${te}1"
+elif ! echo "$da1" |diff - "${tf}1" >"$tdiff"; then
+    kill "$pids"
+    $PRINTF "$NO_RESULT (first conn failed); diff:\n"
+    cat "$tdiff"
+else
+echo "$da2" |eval "$CLI" >"${tf}2" 2>"${te}2"
+rc="$?"; kill "$pids"
+if [ $rc -ne 0 ]; then
+    $PRINTF "$FAILED:\n"
+    echo "$SRV &"
+    echo "$CLI"
+    cat "${te}s" "${te}2"
+elif ! echo "$da2" |diff - "${tf}2" >"$tdiff"; then
+    $PRINTF "$FAILED: diff\n"
+    cat "$tdiff"
+else
+    $PRINTF "$OK\n"
+fi
+fi
+fi
+esac
+N=$((N+1))
+
 #==============================================================================
 
 rm -f testsrv.* testcli.* testsrvdsa*
@@ -3318,6 +3442,9 @@ rm -f testsrv.* testcli.* testsrvdsa*
 # file systems 
 #rm -r "$td"
 
+# sometimes subprocesses hang; we want to see this
+wait
+
 exit
 
 # template
