@@ -1,4 +1,4 @@
-/* $Id: xio-creat.c,v 1.10 2005/03/13 12:21:03 gerhard Exp $ */
+/* $Id: xio-creat.c,v 1.12 2005/09/03 16:51:36 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -42,6 +42,7 @@ static int xioopen_creat(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
    char *a2;
    struct opt *opts = NULL;
    bool exists;
+   bool opt_unlink_close = false;
    int result;
 
    if ((a2 = strchr(a1, ',')) != NULL) {
@@ -52,6 +53,15 @@ static int xioopen_creat(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
    if ((result = _xioopen_named_early(a1, a2, groups, &exists, &opts)) < 0) {
       return result;
    }
+   if (applyopts_single(&fd->stream, opts, PH_INIT) < 0)  return -1;
+
+   retropt_bool(opts, OPT_UNLINK_CLOSE, &opt_unlink_close);
+   if (opt_unlink_close) {
+      if ((fd->stream.unlink_close = strdup(a1)) == NULL) {
+	 Error1("strdup(\"%s\"): out of memory", a1);
+      }
+      fd->stream.opt_unlink_close = true;
+   }
 
    Notice2("creating regular file \"%s\" for %s", a1, ddirection[rw]);
    if ((result = _xioopen_creat(a1, rw, opts)) < 0)
