@@ -1,4 +1,4 @@
-/* $Id: xio-ipapp.c,v 1.19 2005/03/13 12:15:33 gerhard Exp $ */
+/* $Id: xio-ipapp.c,v 1.21 2005/08/18 19:51:25 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -49,7 +49,7 @@ int xioopen_ip4app_connect(char *a1, int xioflags, xiofile_t *xxfd,
    }
 
    applyopts(-1, opts, PH_INIT);
-   applyopts_single(xfd, opts, PH_INIT);
+   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
 
    retropt_bool(opts, OPT_FORK, &dofork);
 
@@ -117,6 +117,14 @@ int xioopen_ip4app_connect(char *a1, int xioflags, xiofile_t *xxfd,
 	 }
 	 if (pid == 0) {	/* child process */
 	    Info1("just born: TCP client process "F_pid, Getpid());
+
+	    xiodroplocks();	/* drop locks in xio addresses */
+	    /* some locks belong to parent, so "drop" them now */
+	    if (xiohook_newchild) {
+	       if ((*xiohook_newchild)(pid) != 0) {
+		  Exit(1);
+	       }
+	    }
 	    break;
 	 }
 	 /* parent process */
@@ -245,7 +253,6 @@ int _xioopen_ip4app_listen_prepare(struct opt *opts, struct opt **opts0,
    retropt_int(opts, OPT_SO_TYPE, socktype);
 
    *opts0 = copyopts(opts, GROUP_ALL);
-
    return STAT_OK;
 }
 
@@ -265,7 +272,7 @@ int xioopen_ip4app_listen(char *a1, int xioflags, xiofile_t *fd,
    if (parseopts(a2, groups, &opts) < 0)
       return STAT_NORETRY;
    applyopts(-1, opts, PH_INIT);
-   applyopts_single(&fd->stream, opts, PH_INIT);
+   if (applyopts_single(&fd->stream, opts, PH_INIT) < 0)  return -1;
 
    if (_xioopen_ip4app_listen_prepare(opts, &opts0, a1, protname, us, &socktype)
        != STAT_OK) {
@@ -273,9 +280,9 @@ int xioopen_ip4app_listen(char *a1, int xioflags, xiofile_t *fd,
    }
 
    if ((result =
-	_xioopen_listen(&fd->stream, xioflags,
+	xioopen_listen(&fd->stream, xioflags,
 		     (struct sockaddr *)us, sizeof(struct sockaddr_in),
-		     opts, PF_INET, socktype, ipproto, E_ERROR))
+		     opts, opts0, PF_INET, socktype, ipproto))
        != 0)
       return result;
    return 0;
@@ -318,7 +325,7 @@ int xioopen_ip6app_connect(char *a1, int xioflags, xiofile_t *xxfd,
       return STAT_NORETRY;
    }
    applyopts(-1, opts, PH_INIT);
-   applyopts_single(xfd, opts, PH_INIT);
+   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
 #if WITH_RETRY
    ++xfd->retry;
 #endif /* WITH_RETRY */
@@ -390,6 +397,14 @@ int xioopen_ip6app_connect(char *a1, int xioflags, xiofile_t *xxfd,
 	 }
 	 if (pid == 0) {	/* child process */
 	    Info1("just born: TCP client process "F_pid, Getpid());
+
+	    xiodroplocks();	/* drop locks in xio addresses */
+	    /* some locks belong to parent, so "drop" them now */
+	    if (xiohook_newchild) {
+	       if ((*xiohook_newchild)(pid) != 0) {
+		  Exit(1);
+	       }
+	    }
 	    break;
 	 }
 	 /* parent process */
