@@ -1,4 +1,4 @@
-/* $Id: xio-listen.h,v 1.8 2005/03/13 12:15:59 gerhard Exp $ */
+/* $Id: xio-listen.h,v 1.9 2005/07/30 15:50:12 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -10,6 +10,11 @@ extern const struct optdesc opt_fork;
 extern const struct optdesc opt_range;
 extern const struct optdesc opt_tcpwrappers;
 
+int
+   xioopen_listen(struct single *xfd, int xioflags,
+		  struct sockaddr *us, socklen_t uslen,
+		  struct opt *opts, struct opt *opts0,
+		  int pf, int socktype, int proto);
 int _xioopen_listen(struct single *fd, int xioflags,
 		    struct sockaddr *us, socklen_t uslen,
 		 struct opt *opts, int pf, int socktype, int proto, int level);
