@@ -1,4 +1,4 @@
-/* $Id: xio-pipe.c,v 1.17 2005/03/13 12:10:01 gerhard Exp $ */
+/* $Id: xio-pipe.c,v 1.19 2005/09/03 16:51:36 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -31,6 +31,8 @@ int xioopen_fifo_unnamed(char *arg, xiofile_t *sock) {
    if (parseopts(arg, GROUP_FD|GROUP_FIFO, &opts) < 0)
       return STAT_NORETRY;
    applyopts(-1, opts, PH_INIT);
+   if (applyopts_single(&sock->stream, opts, PH_INIT) < 0)  return -1;
+
    if (Pipe(filedes) != 0) {
       Error2("pipe(%p): %s", filedes, strerror(errno));
       return -1;
@@ -88,7 +90,8 @@ static int xioopen_fifo(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
 #else
    struct stat pipstat;
 #endif /* !HAVE_STAT64 */
-   bool do_unlink_early = false;
+   bool opt_unlink_early = false;
+   bool opt_unlink_close = true;
    mode_t mode = 0666;
    int result;
    char *a2;
@@ -99,9 +102,10 @@ static int xioopen_fifo(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
    if ((result = parseopts(a2, groups, &opts)) < 0)
       return result;
    applyopts(-1, opts, PH_INIT);
+   if (applyopts_single(&fd->stream, opts, PH_INIT) < 0)  return -1;
 
-   retropt_bool(opts, OPT_UNLINK_EARLY, &do_unlink_early);
-   if (do_unlink_early) {
+   retropt_bool(opts, OPT_UNLINK_EARLY, &opt_unlink_early);
+   if (opt_unlink_early) {
       if (Unlink(a1) < 0) {
 	 if (errno == ENOENT) {
 	    Warn2("unlink(%s): %s", a1, strerror(errno));
@@ -112,6 +116,7 @@ static int xioopen_fifo(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
       }
    }
 
+   retropt_bool(opts, OPT_UNLINK_CLOSE, &opt_unlink_close);
    retropt_modet(opts, OPT_PERM, &mode);
    if (applyopts_named(a1, opts, PH_EARLY) < 0) {
       return STAT_RETRYLATER;
@@ -145,6 +150,12 @@ static int xioopen_fifo(char *a1, int xioflags, xiofile_t *fd, unsigned groups,
 #endif
 	 Notice2("created named pipe \"%s\" for %s", a1, ddirection[rw]);
       }
+      if (opt_unlink_close) {
+	 if ((fd->stream.unlink_close = strdup(a1)) == NULL) {
+	    Error1("strdup(\"%s\"): out of memory", a1);
+	 }
+	 fd->stream.opt_unlink_close = true;
+      }
    } else {
       /* exists */
       Debug1("xioopen_fifo(\"%s\"): already exist, opening it", a1);
