@@ -1,4 +1,4 @@
-/* $Id: xio-proxy.c,v 1.18 2005/03/13 12:16:32 gerhard Exp $ */
+/* $Id: xio-proxy.c,v 1.19 2005/08/18 19:51:25 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2002-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -118,7 +118,7 @@ static int xioopen_proxy_connect(char *a1, int xioflags, xiofile_t *xxfd,
    }
 
    applyopts(-1, opts, PH_INIT);
-   applyopts_single(xfd, opts, PH_INIT);
+   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
 
    retropt_int(opts, OPT_SO_TYPE, &socktype);
 
@@ -206,6 +206,14 @@ static int xioopen_proxy_connect(char *a1, int xioflags, xiofile_t *xxfd,
 	 }
 	 if (pid == 0) {	/* child process */
 	    Info1("just born: proxy client process "F_pid, Getpid());
+
+	    xiodroplocks();	/* drop locks in xio addresses */
+	    /* some locks belong to parent, so "drop" them now */
+	    if (xiohook_newchild) {
+	       if ((*xiohook_newchild)(pid) != 0) {
+		  Exit(1);
+	       }
+	    }
 	    xfd->forever = false;  xfd->retry = 0;
 	    break;
 	 }
