@@ -1,4 +1,4 @@
-/* $Id: xio-pty.c,v 1.13 2005/03/18 20:24:39 gerhard Exp $ */
+/* $Id: xio-pty.c,v 1.15 2005/09/03 16:51:36 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2002-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -41,6 +41,7 @@ static int xioopen_pty(char *a1, int xioflags, xiofile_t *xfd, unsigned groups,
    char ptyname[MAXPTYNAMELEN];
    char *tn = NULL;
    char *linkname = NULL;
+   bool opt_unlink_close = true;	/* remove symlink afterwards */
    bool wait_slave = false;	/* true would be better for many platforms, but
 				   some OSes cannot handle this, and for common
 				   default behaviour as well as backward 
@@ -50,6 +51,9 @@ static int xioopen_pty(char *a1, int xioflags, xiofile_t *xfd, unsigned groups,
    if (parseopts(a1, groups, &opts) < 0)
       return STAT_NORETRY;
    applyopts(-1, opts, PH_INIT);
+   if (applyopts_single(&xfd->stream, opts, PH_INIT) < 0)  return -1;
+
+   retropt_bool(opts, OPT_UNLINK_CLOSE, &opt_unlink_close);
 
    /* trying to set user-early, perm-early etc. here might be useless because
       file system entry is eventually available only past pty creation */
@@ -80,6 +84,7 @@ static int xioopen_pty(char *a1, int xioflags, xiofile_t *xfd, unsigned groups,
 #endif /* HAVE_POLL */
 
    applyopts2(-1, opts, PH_INIT, PH_EARLY);
+   if (applyopts_single(&xfd->stream, opts, PH_INIT) < 0)  return -1;
 
    applyopts(-1, opts, PH_PREBIGEN);
 
@@ -146,6 +151,12 @@ static int xioopen_pty(char *a1, int xioflags, xiofile_t *xfd, unsigned groups,
 	 Error3("symlink(\"%s\", \"%s\"): %s",
 		ptyname, linkname, strerror(errno));
       }
+      if (opt_unlink_close) {
+	 if ((xfd->stream.unlink_close = strdup(linkname)) == NULL) {
+	    Error1("strdup(\"%s\"): out of memory", linkname);
+	 }
+	 xfd->stream.opt_unlink_close = true;
+      }
    }
 
    applyopts_named(ptyname, opts, PH_PASTOPEN);
@@ -159,7 +170,7 @@ static int xioopen_pty(char *a1, int xioflags, xiofile_t *xfd, unsigned groups,
    
    xfd->stream.fd = ptyfd;
    applyopts(ptyfd, opts, PH_LATE);
-   applyopts_single(&xfd->stream, opts, PH_LATE);
+   if (applyopts_single(&xfd->stream, opts, PH_LATE) < 0)  return -1;
 
 #if HAVE_POLL
    /* if you can and wish: */
