@@ -1,5 +1,5 @@
-/* $Id: xio-socket.c,v 1.29 2005/03/18 20:25:29 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2005 */
+/* $Id: xio-socket.c,v 1.31 2006/01/29 16:36:58 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for socket related functions */
@@ -246,7 +246,13 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
       fcntl_flags = Fcntl(xfd->fd, F_GETFL);
       Fcntl_l(xfd->fd, F_SETFL, fcntl_flags|O_NONBLOCK);
    }
-   if ((result = Connect(xfd->fd, (struct sockaddr *)them, themlen)) < 0) {
+
+   result = Connect(xfd->fd, (struct sockaddr *)them, themlen);
+   if (Getsockname(xfd->fd, &la.soa, &lalen) < 0) {
+      Warn4("getsockname(%d, %p, {%d}): %s",
+	    xfd->fd, &la.soa, lalen, strerror(errno));
+   }
+   if (result < 0) {
       if (errno == EINPROGRESS) {
 	 if (xfd->para.socket.connect_timeout.tv_sec  != 0 ||
 	     xfd->para.socket.connect_timeout.tv_usec != 0) {
@@ -312,11 +318,6 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
       }
    }
 
-   if (Getsockname(xfd->fd, &la.soa, &lalen) < 0) {
-      Warn4("getsockname(%d, %p, {%d}): %s",
-	    xfd->fd, &la.soa, lalen, strerror(errno));
-   }
-
    xfd->howtoend = END_SHUTDOWN;
 
    applyopts_fchown(xfd->fd, opts);
@@ -401,6 +402,14 @@ int xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 	 }
 	 if (pid == 0) {	/* child process */
 	    Info1("just born: TCP client process "F_pid, Getpid());
+
+	    xiodroplocks();	/* drop locks in xio addresses */
+	    /* some locks belong to parent, so "drop" them now */
+	    if (xiohook_newchild) {
+	       if ((*xiohook_newchild)(pid) != 0) {
+		  Exit(1);
+	       }
+	    }
 	    break;
 	 }
 	 /* parent process */
