@@ -1,4 +1,4 @@
-/* $Id: xio-socks.c,v 1.23 2005/03/13 12:17:34 gerhard Exp $ */
+/* $Id: xio-socks.c,v 1.24 2005/08/18 19:51:25 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -80,7 +80,7 @@ static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *xxfd,
    socket_in_init(us);
 
    applyopts(-1, opts, PH_INIT);
-   applyopts_single(xfd, opts, PH_INIT);
+   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
 
    retropt_int(opts, OPT_SO_TYPE, &socktype);
 
@@ -184,6 +184,14 @@ static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *xxfd,
 	 }
 	 if (pid == 0) {	/* child process */
 	    Info1("just born: socks client process "F_pid, Getpid());
+
+	    xiodroplocks();	/* drop locks in xio addresses */
+	    /* some locks belong to parent, so "drop" them now */
+	    if (xiohook_newchild) {
+	       if ((*xiohook_newchild)(pid) != 0) {
+		  Exit(1);
+	       }
+	    }
 	    xfd->forever = false;  xfd->retry = 0;
 	    break;
 	 }
