@@ -1,5 +1,5 @@
-/* $Id: xio-system.c,v 1.10 2003/12/07 19:58:00 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2003 */
+/* $Id: xio-system.c,v 1.11 2005/03/13 12:18:03 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of system type */
@@ -14,7 +14,7 @@
 #if WITH_SYSTEM
 
 static int xioopen_system(char *arg,
-		int rw,	/* O_RDONLY etc. */
+		int xioflags,	/* XIO_RDONLY etc. */
 		xiofile_t *fd,
 		unsigned groups,
 		int dummy1, int dummy2, void *dummyp1
@@ -24,7 +24,7 @@ const struct addrdesc addr_system = { "system", 3, xioopen_system, GROUP_FD|GROU
 
 
 static int xioopen_system(char *arg,
-		int rw,	/* O_RDONLY etc. */
+		int xioflags,	/* XIO_RDONLY etc. */
 		xiofile_t *fd,
 		unsigned groups,
 		int dummy1, int dummy2, void *dummyp1
@@ -43,7 +43,7 @@ static int xioopen_system(char *arg,
    if (parseopts(comma, groups, &opts) < 0)
       return STAT_NORETRY;
 
-   status = _xioopen_foxec(comma, rw, &fd->stream, groups, &opts);
+   status = _xioopen_foxec(comma, xioflags, &fd->stream, groups, &opts);
    if (status < 0)  return status;
    if (status == 0) {	/* child */
       int numleft;
