@@ -1,4 +1,4 @@
-/* $Id: xio-tcp.c,v 1.18 2005/03/13 12:18:12 gerhard Exp $ */
+/* $Id: xio-tcp.c,v 1.20 2005/08/18 19:52:32 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -115,7 +115,7 @@ int xioopen_tcp6_listen(char *a1, int xioflags, xiofile_t *fd,
 			     unsigned groups, int dummy1, int dummy2,
 			     void *dummyp1) {
    char *a2;
-   struct opt *opts = NULL;
+   struct opt *opts = NULL, *opts0;
    struct sockaddr_in6 us;
    struct sockaddr_in6 them;
    int needbind = 0;
@@ -142,19 +142,15 @@ int xioopen_tcp6_listen(char *a1, int xioflags, xiofile_t *fd,
       needbind = true;
 
    applyopts(-1, opts, PH_INIT);
-   applyopts_single(&fd->stream, opts, PH_INIT);
+   if (applyopts_single(&fd->stream, opts, PH_INIT) < 0)  return -1;
    retropt_int(opts, OPT_SO_TYPE, &socktype);
 
+   opts0 = copyopts(opts, GROUP_ALL);
+
    if ((result =
-	_xioopen_listen(&fd->stream, xioflags,
+	xioopen_listen(&fd->stream, xioflags,
 			(struct sockaddr *)&us, sizeof(struct sockaddr_in6),
-			opts, PF_INET6, socktype, IPPROTO_TCP,
-#if WITH_RETRY
-			(fd->stream.retry||fd->stream.forever)?E_INFO:E_ERROR
-#else
-			E_ERROR
-#endif /* WITH_RETRY */
-			))
+		       opts, opts0, PF_INET6, socktype, IPPROTO_TCP))
        != 0)
       return result;
    return 0;
