@@ -1,4 +1,4 @@
-/* $Id: xio-unix.c,v 1.17 2005/03/13 12:18:29 gerhard Exp $ */
+/* $Id: xio-unix.c,v 1.20 2005/09/04 09:42:49 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -19,10 +19,10 @@ static int xioopen_unix_connect(char *a1, int xioflags, xiofile_t *fd, unsigned
 static int xioopen_unix_listen(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1);
 
 
-const struct addrdesc addr_unix_connect = { "unix-connect",   3, xioopen_unix_connect, GROUP_FD|GROUP_NAMED|GROUP_SOCKET|GROUP_SOCK_UNIX, 0, 0, NULL HELP(":<filename>") };
+const struct addrdesc addr_unix_connect = { "unix-connect",   3, xioopen_unix_connect, GROUP_FD|GROUP_NAMED|GROUP_SOCKET|GROUP_SOCK_UNIX|GROUP_RETRY, 0, 0, NULL HELP(":<filename>") };
 
 #if WITH_LISTEN
-const struct addrdesc addr_unix_listen  = { "unix-listen", 3, xioopen_unix_listen, GROUP_FD|GROUP_NAMED|GROUP_SOCKET|GROUP_SOCK_UNIX|GROUP_LISTEN|GROUP_CHILD, 0, 0, NULL HELP(":<filename>") };
+const struct addrdesc addr_unix_listen  = { "unix-listen", 3, xioopen_unix_listen, GROUP_FD|GROUP_NAMED|GROUP_SOCKET|GROUP_SOCK_UNIX|GROUP_LISTEN|GROUP_CHILD|GROUP_RETRY, 0, 0, NULL HELP(":<filename>") };
 #endif /* WITH_LISTEN */
 
 
@@ -31,9 +31,10 @@ static int xioopen_unix_listen(char *a1, int xioflags, xiofile_t *fd, unsigned g
    /* we expect the form: filename */
    struct sockaddr_un us;
    char *name;
+   struct opt *opts = NULL, *opts0 = NULL;
    int socktype = SOCK_STREAM;
-   bool do_unlink_early = false;
-   struct opt *opts = NULL;
+   bool opt_unlink_early = false;
+   bool opt_unlink_close = true;
    int result;
 
    socket_un_init(&us);
@@ -56,9 +57,10 @@ static int xioopen_unix_listen(char *a1, int xioflags, xiofile_t *fd, unsigned g
    if (parseopts(a1, groups, &opts) < 0)
       return STAT_NORETRY;
    applyopts(-1, opts, PH_INIT);
+   if (applyopts_single(&fd->stream, opts, PH_INIT) < 0)  return -1;
 
-   retropt_bool(opts, OPT_UNLINK_EARLY, &do_unlink_early);
-   if (do_unlink_early) {
+   retropt_bool(opts, OPT_UNLINK_EARLY, &opt_unlink_early);
+   if (opt_unlink_early) {
       if (Unlink(name) < 0) {
 	 if (errno == ENOENT) {
 	    Warn2("unlink(\"%s\"): %s", name, strerror(errno));
@@ -68,17 +70,27 @@ static int xioopen_unix_listen(char *a1, int xioflags, xiofile_t *fd, unsigned g
       }
    }
 
+   retropt_bool(opts, OPT_UNLINK_CLOSE, &opt_unlink_close);
+
    /* trying to set user-early, perm-early etc. here is useless because
       file system entry is available only past bind() call. */
-
    applyopts_named(name, opts, PH_EARLY);	/* umask! */
 
    retropt_int(opts, OPT_SO_TYPE, &socktype);
 
+   if (opt_unlink_close) {
+      if ((fd->stream.unlink_close = strdup(name)) == NULL) {
+	 Error1("strdup(\"%s\"): out of memory", name);
+      }
+      fd->stream.opt_unlink_close = true;
+   }
+
+   opts0 = copyopts(opts, GROUP_ALL);
+
    if ((result =
-	_xioopen_listen(&fd->stream, xioflags,
+	xioopen_listen(&fd->stream, xioflags,
 		     (struct sockaddr *)&us, sizeof(struct sockaddr_un),
-		     opts, PF_UNIX, socktype, 0, E_ERROR))
+		     opts, opts0, PF_UNIX, socktype, 0))
        != 0)
       return result;
    return 0;
@@ -86,12 +98,14 @@ static int xioopen_unix_listen(char *a1, int xioflags, xiofile_t *fd, unsigned g
 #endif /* WITH_LISTEN */
 
 
-static int xioopen_unix_connect(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
+static int xioopen_unix_connect(char *a1, int xioflags, xiofile_t *xxfd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
    /* we expect the form: filename */
+   struct single *xfd = &xxfd->stream;
    char *a2;
    bool needbind = false;
    struct sockaddr_un them, us;
    struct opt *opts = NULL;
+   bool opt_unlink_close = false;
    int result;
 
    socket_un_init(&us);
@@ -104,21 +118,31 @@ static int xioopen_unix_connect(char *a1, int xioflags, xiofile_t *fd, unsigned
    if (parseopts(a2, groups, &opts) < 0) {
       return STAT_NORETRY;
    }
+   retropt_bool(opts, OPT_UNLINK_CLOSE, &opt_unlink_close);
+
    applyopts(-1, opts, PH_INIT);
+   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
 
    if (retropt_bind(opts, AF_UNIX, NULL, (struct sockaddr *)&us, 0) >= 0) {
       needbind = true;
    }
 
+   if (opt_unlink_close) {
+      if ((xxfd->stream.unlink_close = strdup(a1)) == NULL) {
+	 Error1("strdup(\"%s\"): out of memory", a1);
+      }
+      xxfd->stream.opt_unlink_close = true;
+   }
+
    if ((result =
-	xioopen_connect(&fd->stream,
+	xioopen_connect(xfd,
 		      needbind?(struct sockaddr *)&us:NULL, sizeof(us),
 		      (struct sockaddr *)&them,
 		      sizeof(struct sockaddr_un),
 		      opts, PF_UNIX, SOCK_STREAM, 0, false)) != 0) {
       return result;
    }
-   if ((result = _xio_openlate(&fd->stream, opts)) < 0) {
+   if ((result = _xio_openlate(xfd, opts)) < 0) {
       return result;
    }
    return 0;
