@@ -1,5 +1,5 @@
-/* $Id: xioclose.c,v 1.19 2004/09/18 12:13:14 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xioclose.c,v 1.23 2006/01/29 16:34:55 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2006 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended close function */
@@ -7,6 +7,7 @@
 
 #include "xiosysincludes.h"
 #include "xioopen.h"
+#include "xiolockfile.h"
 
 #include "xio-termios.h"
 
@@ -15,7 +16,7 @@
 int xioclose1(struct single *pipe) {
 
    if (pipe->tag == XIO_TAG_INVALID) {
-      Error("xioclose1(): invalid file descriptor");
+      Notice("xioclose1(): invalid file descriptor");
       errno = EINVAL;
       return -1;
    }
@@ -28,10 +29,17 @@ int xioclose1(struct single *pipe) {
 #endif /* WITH_READLINE */
 #if WITH_OPENSSL
    if (pipe->dtype == DATA_OPENSSL) {
-      sycSSL_shutdown(pipe->para.openssl.ssl);
-      Close(pipe->fd);
-      sycSSL_free(pipe->para.openssl.ssl);
-      sycSSL_CTX_free(pipe->para.openssl.ctx);
+      if (pipe->para.openssl.ssl) {
+	 /* e.g. on TCP connection refused, we do not yet have this set */
+	 sycSSL_shutdown(pipe->para.openssl.ssl);
+	 sycSSL_free(pipe->para.openssl.ssl);
+	 pipe->para.openssl.ssl = NULL;
+      }
+      Close(pipe->fd);  pipe->fd = -1;
+      if (pipe->para.openssl.ctx) {
+	 sycSSL_CTX_free(pipe->para.openssl.ctx);
+	 pipe->para.openssl.ctx = NULL;
+      }
    } else
 #endif /* WITH_OPENSSL */
 #if WITH_TERMIOS
@@ -70,10 +78,24 @@ int xioclose1(struct single *pipe) {
       case END_NONE: default: break;
       }
    }
+
+   /* unlock */
+   if (pipe->havelock) {
+      xiounlock(pipe->lock.lockfile);
+      pipe->havelock = false;
+   }      
+   if (pipe->opt_unlink_close && pipe->unlink_close) {
+      if (Unlink(pipe->unlink_close) < 0) {
+	 Info2("unlink(\"%s\"): %s", pipe->unlink_close, strerror(errno));
+      }
+      free(pipe->unlink_close);
+   }
+
    pipe->tag = XIO_TAG_INVALID;
    return 0;	/*! */
 }
 
+
 /* close the xio fd */
 int xioclose(xiofile_t *file) {
    int result;
