@@ -1,4 +1,4 @@
-/* $Id: xioopts.h,v 1.51 2005/03/12 18:02:50 gerhard Exp $ */
+/* $Id: xioopts.h,v 1.54 2005/09/03 16:51:36 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -29,7 +29,7 @@ enum e_types {
    TYPE_FILENAME = TYPE_STRING,
    TYPE_PTRDIFF,	/* ptrdiff_t */
    TYPE_SHORT,		/* short */
-   TYPE_SIZET,		/* size_t */
+   TYPE_SIZE_T,		/* size_t */
    TYPE_SOCKADDR,	/* struct sockaddr * */
    TYPE_UINT,		/* unsigned int */
    TYPE_ULONG,		/* unsigned long */
@@ -333,6 +333,7 @@ enum e_optcode {
    OPT_IXANY,		/* termios.c_iflag */
    OPT_IXOFF,		/* termios.c_iflag */
    OPT_IXON,		/* termios.c_iflag */
+   OPT_LOCKFILE,
    OPT_LOWPORT,
 #ifdef NLDLY
 #  ifdef NL0
@@ -459,6 +460,7 @@ enum e_optcode {
    OPT_PTY_WAIT_SLAVE,
    OPT_RANGE,		/* restrict client socket address */
    OPT_RAW,		/* termios */
+   OPT_READBYTES,
    OPT_RETRY,
    OPT_SANE,		/* termios */
    OPT_SEEK32_CUR,
@@ -653,6 +655,7 @@ enum e_optcode {
    OPT_TOSTOP,		/* termios.c_lflag */
    OPT_UMASK,
    OPT_UNLINK,
+   OPT_UNLINK_CLOSE,
    OPT_UNLINK_EARLY,
    OPT_UNLINK_LATE,
    OPT_USER,
@@ -689,6 +692,7 @@ enum e_optcode {
 #ifdef VWERASE
    OPT_VWERASE,		/* termios.c_cc */
 #endif
+   OPT_WAITLOCK,
 #ifdef XCASE
    OPT_XCASE,		/* termios.c_lflag */
 #endif
